unit DDef;

interface

uses
  DUtils,
  DMath;

const
  // particle flags
  PF_Line       = 1; // render [circle/line]
  PF_Move       = 2; // [not/do] use Dir as velocity 
  PF_Light      = 4; // is the particle is light?
  PF_Collide    = 8; // collide with other objects?
  PF_Killed     = 16;

type
TParticle = packed record
  Flags: Cardinal;
  Pos: TVec2f; // position
  Dir: TVec2f; // direction
  Rad: Single; // radius
  Col: TVec4f; // color
  ID: Cardinal;
end;

TParticleSystem = class
private
  FLastUpdate: Integer;
  FParticles: array of TParticle;
  procedure AllocParticles(Count: Integer);
  function FindKilled: Integer;
public
  constructor Create;
  procedure Render; 
  procedure Update;
  function CreateParticle(Flags: Cardinal; 
                          Pos, Dir: TVec2f;
                          Rad: Single;
                          Col: TVec4f;
                          ID: Cardinal): Integer;
  function GetParticle(Index: Integer): TParticle;
  procedure SetParticle(Index: Integer; Particle: TParticle);
  procedure KillParticle(Index: Integer);
  property Particles[Index: Integer]: TParticle read GetParticle write SetParticle; default;
end;

implementation

// {{{ TParticleSystem
procedure TParticleSystem.AllocParticles(Count: Integer);//{{{
  var
    I: Integer;
begin
  SetLength(FParticles, Length(FParticles) + Count);
  for I := Length(FParticles) - Count to High(FParticles) do
    FParticles[I].Flags := PF_Killed;
end;
//}}}
function TParticleSystem.FindKilled: Integer;//{{{
begin
  Result := 0;
  while (Result < Length(FParticles)) and 
        (FParticles[Result].Flags and PF_Killed > 0) do
    Inc(Result);
  if Result = Length(FParticles) then
    AllocParticles(10);
end;
//}}}
constructor TParticleSystem.Create;//{{{
  var
    I: Integer;
begin
  SetLength(FParticles, 0);
  AllocParticles(10);
  FLastUpdate := -1;
end;
//}}}
procedure TParticleSystem.Render;//{{{
  var
    I: Integer;
begin
  for I := 0 to High(FParticles) do
    with FParticles[I] do
      if (Flags and PF_Killed = 0) and (Flags and PF_Killed > 0) then begin
        // TODO: Lighting
      end;
  for I := 0 to High(FParticles) do
    with FParticles[I] do begin
      Game.Color4f(Col);
      if Flags and PF_Line > 0 then begin
        LineWidth(Rad);
        DrawLine(Pos, Pos + Dir)
      end else
        DrawCircle(Pos, Rad);
    end;
end;
//}}}
procedure TParticleSystem.Update;//{{{
  var
    I: Integer;
    Delta: Integer;
begin
  Delta := GetTimer - FLastUpdate;
  if Delta > 500 then begin
    Delta := 500; 
    FLastUpdate := GetTimer;
  end;
  for I := 0 to High(FParticles) do
    if FParticles[I].Flags and PF_Killed = 0 then begin
      if FParticles[I].Flags and PF_Move > 0 then
        FParticles[I].Pos += FParticles[I].Dir * Delta/1000;
      if FParticles[I].Flags and PF_Collide > 0 then
        {TODO:  };
    end;
end;
//}}}
function TParticeSystem.CreateParticle;//{{{
begin
  Result := FindKilled;
  FParticles[Result].Flags := Flags;
  FParticles[Result].Pos := Pos;
  FParticles[Result].Dir := Dir;
  FParticles[Result].Rad := Rad;
  FParticles[Result].Col := Col;
  FParticles[Result].ID := ID;
end;
//}}}
function TParticleSystem.GetParticle(Index: Integer): TParticle;//{{{
begin
  Result := FParticles[Index];
end;
//}}}
procedure TParticleSystem.SetParticle(Index: Integer; Particle: TParticle);//{{{
begin
  FParticles[Index] := Particle;
end;
//}}}
procedure TParticleSystem.KillParticle(Index: Integer);//{{{
begin
  FPartices[Index].Flags := PF_Killed;
end;
//}}}
//}}}

end.
