unit DMath;

interface

const
  EPS = 10e-6;

function Min(A, B: Integer): Integer; overload;
function Max(A, B: Integer): Integer; overload;
function Min(A, B: Single): Single; overload;
function Max(A, B: Single): Single; overload

type
TVec2i = packed record
  case integer of
    0: (X, Y: integer);
    1: (Arr: array[0..1] of integer);
end;

TVec2f = packed record
  case integer of
    0: (X, Y: Single);
    1: (Arr: array[0..1] of Single);
end;

TVec4f = packed record
  case integer of
    0: (X, Y, Z, W: Single);
    1: (Arr: array[0..3] of Single);
end;

TRectf = record
  A, B: TVec2f;
end;
TRect = TRectf;

TLine2f = record
  N: TVec2f;
  D: Single;
end;

function Vec2i(X, Y: Integer): TVec2i;
function Round2f(V: TVec2f): TVec2i;
function Add2i(A, B: TVec2i): TVec2i;
function Sub2i(A, B: TVec2i): TVec2i;
function Mul2i(A: TVec2i; I: integer): TVec2i; overload;
function Mul2i(A: TVec2i; S: single): TVec2f; overload;
function Len2i(A: TVec2i): Single;

function Vec2f(X, Y: Single): TVec2f; overload;
function Vec2f(V: TVec2i): TVec2f; overload;
function Add2f(A, B: TVec2f): TVec2f;
function Sub2f(A, B: TVec2f): TVec2f;
function Mul2f(A: TVec2f; S: single): TVec2f;
function Len2f(A: TVec2f): Single;
function Dot2f(A, B: TVec2f): Single;
function Norm2f(V: TVec2f): TVec2f;

function Vec4f(X, Y, Z, W: Single): TVec4f;
function Add4f(A, B: TVec4f): TVec4f;
function Sub4f(A, B: TVec4f): TVec4f;
function Mul4f(A: TVec4f; S: Single): TVec4f;

function Rect(A, B: TVec2f): TRect;

function Line2f(N: TVec2f; D: Single): TLine2f; overload;
function Line2f(A, B: TVec2f): TLine2f; overload;
function Dist2f(Line: TLine2f; P: TVec2f): Single; overload;

{$IFDEF OPMATH}
operator = (A, B: TVec2i) Result: Boolean; inline;
operator := (A: TVec2i) Result: TVec2f; inline;
operator + (A, B: TVec2i) Result: TVec2i; inline;
operator + (A, B: TVec2f) Result: TVec2f; inline;
operator + (A, B: TVec4f) Result: TVec4f; inline;
operator - (A, B: TVec2i) Result: TVec2i; inline;
operator - (A, B: TVec2f) Result: TVec2f; inline;
operator - (A, B: TVec4f) Result: TVec4f; inline;
operator * (I: integer; A: TVec2i) Result: TVec2i; inline;
operator * (A: TVec2i; I: Integer) Result: TVec2i; inline;
operator * (S: Single; A: TVec2f) Result: TVec2f; inline;
operator * (A: TVec2f; S: Single) Result: TVec2f; inline;
operator * (A: TVec4f; S: Single) Result: TVec4f; inline;
operator * (S: Single; A: TVec4f) Result: TVec4f; inline;
operator div (A: TVec2i; I: Integer) Result: TVec2i; inline;
operator / (A: TVec2f; S: Single) Result: TVec2f; inline;
{$ENDIF}

implementation

function Min(A, B: Integer): Integer;
begin
  if A < B then
    Min := A
  else
    Min := B;
end;

function Max(A, B: Integer): Integer;
begin
  if A > B then
    Max := A
  else
    Max := B;
end;

function Min(A, B: Single): Single;
begin
  if A < B then
    Min := A
  else
    Min := B;
end;

function Max(A, B: Single): Single;
begin
  if A > B then
    Max := A
  else
    Max := B;
end;

// TVec2i//{{{
function Vec2i(X, Y: Integer): TVec2i;
begin
  Vec2i.X := X;
  Vec2i.Y := Y;
end;

function Round2f(V: TVec2f): TVec2i;
begin
  Round2f.X := Round(V.X);
  Round2f.Y := Round(V.Y);
end;

function Add2i(A, B: TVec2i): TVec2i;
begin
  Add2i.X := A.X + B.X;
  Add2i.Y := A.Y + B.Y;
end;

function Sub2i(A, B: TVec2i): TVec2i;
begin
  Sub2i.X := A.X - B.X;
  Sub2i.Y := A.Y - B.Y;
end;

function Mul2i(A: TVec2i; I: integer): TVec2i;
begin
  Mul2i.X := I * A.X;
  Mul2i.Y := I * A.Y;
end;

function Mul2i(A: TVec2i; S: single): TVec2f;
begin
  Mul2i.X := S * A.X;
  Mul2i.Y := S * A.Y;
end;

function Len2i(A: TVec2i): Single;
begin
  Len2i := Sqrt(Sqr(A.X) + Sqr(A.Y));
end;
//}}}
// TVec2f//{{{
function Vec2f(X, Y: Single): TVec2f;
begin
  Vec2f.X := X;
  Vec2f.Y := Y;
end;

function Vec2f(V: TVec2i): TVec2f;
begin
  Vec2f := Vec2f(V.X, V.Y);
end;

function Add2f(A, B: TVec2f): TVec2f;
begin
  Add2f.X := A.X + B.X;
  Add2f.Y := A.Y + B.Y;
end;

function Sub2f(A, B: TVec2f): TVec2f;
begin
  Sub2f.X := A.X - B.X;
  Sub2f.Y := A.Y - B.Y;
end;

function Mul2f(A: TVec2f; S: single): TVec2f;
begin
  Mul2f.X := S * A.X;
  Mul2f.Y := S * A.Y;
end;

function Len2f(A: TVec2f): Single;
begin
  Len2f := Sqrt(Sqr(A.X) + Sqr(A.Y));
end;

function Dot2f(A, B: TVec2f): Single;
begin
  Dot2f := A.X * B.X  +  A.Y * B.Y;
end;

function Norm2f(V: TVec2f): TVec2f;
begin
  Norm2f := Mul2f(V, 1 / Len2f(V));
end;
//}}}
//{{{ TVec4f
function Vec4f(X, Y, Z, W: Single): TVec4f;
begin
  Vec4f.X := X;
  Vec4f.Y := Y;
  Vec4f.Z := Z;
  Vec4f.W := W;
end;

function Add4f(A, B: TVec4f): TVec4f;
begin
  Add4f.X := A.X + B.X;
  Add4f.Y := A.Y + B.Y;
  Add4f.Z := A.Z + B.Z;
  Add4f.W := A.W + B.W;
end;

function Sub4f(A, B: TVec4f): TVec4f;
begin
  Sub4f.X := A.X - B.X;
  Sub4f.Y := A.Y - B.Y;
  Sub4f.Z := A.Z - B.Z;
  Sub4f.W := A.W - B.W;
end;

function Mul4f(A: TVec4f; S: Single): TVec4f;
begin
  Mul4f.X := A.X * S;
  Mul4f.Y := A.Y * S;
  Mul4f.Z := A.Z * S;
  Mul4f.W := A.W * S;
end;
//}}}
//{{{ TRect
function Rect(A, B: TVec2f): TRect;
begin
  Rect.A := A;
  Rect.B := B;
end;
//}}}
// TLine2f//{{{
function Line2f(N: TVec2f; D: Single): TLine2f;
begin
  Line2f.N := N;
  Line2f.D := D;
end;

function Line2f(A, B: TVec2f): TLine2f;
begin
  Line2f.N := Norm2f(Sub2f(B, A));
  Line2f.N := Vec2f(- Result.N.Y, Result.N.X);
  Line2f.D := - Dot2f(A, Result.N);
end;

function Dist2f(Line: TLine2f; P: TVec2f): Single; overload;
begin
  Dist2f := Dot2f(Line.N, P) + Line.D;
end;
//}}}
// operators//{{{
{$IFDEF OPMATH}
operator = (A, B: TVec2i) Result: Boolean; inline;
begin
  Result := (A.X = B.X) and (A.Y = B.Y);
end;

operator := (A: TVec2i) Result: TVec2f; inline;
begin
  Result := Vec2f(A.X, A.Y)
end;

operator + (A, B: TVec2i) Result: TVec2i; inline;
begin
  Result := Add2i(A, B);
end;

operator + (A, B: TVec2f) Result: TVec2f; inline;
begin
  Result := Add2f(A, B);
end;

operator + (A, B: TVec4f) Result: TVec4f; inline;
begin
  Result := Add4f(A, B);
end;

operator - (A, B: TVec2i) Result: TVec2i; inline;
begin
  Result := Sub2i(A, B);
end;

operator - (A, B: TVec2f) Result: TVec2f; inline;
begin
  Result := Sub2f(A, B);
end;

operator - (A, B: TVec4f) Result: TVec4f; inline;
begin
  Result := Sub4f(A, B);
end;

operator * (I: integer; A: TVec2i) Result: TVec2i; inline;
begin
  Result := Mul2i(A, I)
end;

operator * (A: TVec2i; I: Integer) Result: TVec2i; inline;
begin
  Result := Mul2i(A, I)
end;

operator * (S: Single; A: TVec2f) Result: TVec2f; inline;
begin
  Result := Mul2f(A, S)
end;

operator * (A: TVec2f; S: Single) Result: TVec2f; inline;
begin
  Result := Mul2f(A, S)
end;

operator * (A: TVec4f; S: Single) Result: TVec4f; inline;
begin
  Result := Mul4f(A, S);
end;

operator * (S: Single; A: TVec4f) Result: TVec4f; inline;
begin
  Result := Mul4f(A, S);
end;

operator div (A: TVec2i; I: Integer) Result: TVec2i; inline;
begin
  Result := Vec2i(A.X div I, A.Y div I);
end;

operator / (A: TVec2f; S: Single) Result: TVec2f; inline;
begin
  Result := Mul2f(A, 1/S);
end;
{$ENDIF OPMATH}
//}}}

end.
