{$MACRO ON}
{$MODE FPC}
unit DMath;

interface

{$DEFINE FASTMATH}
{$DEFINE OPMATH}
{$DEFINE MathDir:={$IFDEF FASTMATH}inline;{$ENDIF}}

const
  EPS = 10e-6;

function Min(A, B: Integer): Integer;
function Max(A, B: Integer): Integer;
function Min(A, B: Single): Single;
function Max(A, B: Single): Single;

type
TVec2i = packed record
  case integer of
    0: (X, Y: integer);
    1: (Arr: array[0..1] of integer);
end;

TVec2f = packed record
  case integer of
    0: (X, Y: Single);
    1: (Arr: array[0..1] of Single);
end;

TVec4f = packed record
  case integer of
    0: (X, Y, Z, W: Single);
    1: (Arr: array[0..3] of Single);
end;

TRectf = record
  A, B: TVec2f;
end;
TRect = TRectf;

TLine2f = record
  N: TVec2f;
  D: Single;
end;

function Vec2i(X, Y: Integer): TVec2i; MathDir
function Round2f(V: TVec2f): TVec2i; MathDir
function Add2i(A, B: TVec2i): TVec2i; MathDir
function Sub2i(A, B: TVec2i): TVec2i; MathDir
function Mul2i(A: TVec2i; I: integer): TVec2i; MathDir
function Mul2i(A: TVec2i; S: single): TVec2f; MathDir
function Len2i(A: TVec2i): Single; MathDir

function Vec2f(X, Y: Single): TVec2f; MathDir
function Vec2f(V: TVec2i): TVec2f; MathDir
function Add2f(A, B: TVec2f): TVec2f; MathDir
function Sub2f(A, B: TVec2f): TVec2f; MathDir
function Mul2f(A: TVec2f; S: single): TVec2f; MathDir
function Len2f(A: TVec2f): Single; MathDir
function Dot2f(A, B: TVec2f): Single; MathDir
function Norm2f(V: TVec2f): TVec2f; MathDir

function Vec4f(X, Y, Z, W: Single): TVec4f; MathDir
function Add4f(A, B: TVec4f): TVec4f; MathDir
function Sub4f(A, B: TVec4f): TVec4f; MathDir
function Mul4f(A: TVec4f; S: Single): TVec4f; MathDir

function Rect(A, B: TVec2f): TRect;
function Contain(A, B: TRect): Boolean;
function Intersect(A, B: TRect): Boolean; overload;
function Intersect(R: TRect; X1, X2, Y: Single): Boolean; overload;
// Delta -    R,  R  A  
function Intersect(R: TRect; A: TVec2f; var Delta: TVec2f): Boolean; overload;
function Intersect(R: TRect; A: TVec2f): Boolean; overload;

function Line2f(N: TVec2f; D: Single): TLine2f; overload;
function Line2f(A, B: TVec2f): TLine2f; overload;
function Dist2f(Line: TLine2f; P: TVec2f): Single; overload;
function Intersect(Line: TLine2f; Rect: TRect; 
                   var T: Single): Boolean; overload;
function Intersect(Pos, Dir: TVec2f; Rect: TRect; 
                   var T: Single): Boolean; overload;

{$IFDEF OPMATH}
operator = (A, B: TVec2i) Result: Boolean; inline;
operator := (A: TVec2i) Result: TVec2f; inline;
operator + (A, B: TVec2i) Result: TVec2i; inline;
operator + (A, B: TVec2f) Result: TVec2f; inline;
operator + (A, B: TVec4f) Result: TVec4f; inline;
operator - (A, B: TVec2i) Result: TVec2i; inline;
operator - (A, B: TVec2f) Result: TVec2f; inline;
operator - (A, B: TVec4f) Result: TVec4f; inline;
operator * (I: integer; A: TVec2i) Result: TVec2i; inline;
operator * (A: TVec2i; I: Integer) Result: TVec2i; inline;
operator * (S: Single; A: TVec2f) Result: TVec2f; inline;
operator * (A: TVec2f; S: Single) Result: TVec2f; inline;
operator * (A: TVec4f; S: Single) Result: TVec4f; inline;
operator * (S: Single; A: TVec4f) Result: TVec4f; inline;
operator div (A: TVec2i; I: Integer) Result: TVec2i; inline;
operator / (A: TVec2f; S: Single) Result: TVec2f; inline;
{$ENDIF}

implementation

function Min(A, B: Integer): Integer;
begin
  if A < B then
    Min := A
  else
    Min := B;
end;

function Max(A, B: Integer): Integer;
begin
  if A > B then
    Max := A
  else
    Max := B;
end;

function Min(A, B: Single): Single;
begin
  if A < B then
    Min := A
  else
    Min := B;
end;

function Max(A, B: Single): Single;
begin
  if A > B then
    Max := A
  else
    Max := B;
end;

// TVec2i//{{{
function Vec2i(X, Y: Integer): TVec2i; MathDir
begin
  Vec2i.X := X;
  Vec2i.Y := Y;
end;

function Round2f(V: TVec2f): TVec2i; MathDir
begin
  Round2f.X := Round(V.X);
  Round2f.Y := Round(V.Y);
end;

function Add2i(A, B: TVec2i): TVec2i; MathDir
begin
  Add2i.X := A.X + B.X;
  Add2i.Y := A.Y + B.Y;
end;

function Sub2i(A, B: TVec2i): TVec2i; MathDir
begin
  Sub2i.X := A.X - B.X;
  Sub2i.Y := A.Y - B.Y;
end;

function Mul2i(A: TVec2i; I: integer): TVec2i; MathDir
begin
  Mul2i.X := I * A.X;
  Mul2i.Y := I * A.Y;
end;

function Mul2i(A: TVec2i; S: single): TVec2f; MathDir
begin
  Mul2i.X := S * A.X;
  Mul2i.Y := S * A.Y;
end;

function Len2i(A: TVec2i): Single; MathDir
begin
  Len2i := Sqrt(Sqr(A.X) + Sqr(A.Y));
end;
//}}}
// TVec2f//{{{
function Vec2f(X, Y: Single): TVec2f; MathDir
begin
  Vec2f.X := X;
  Vec2f.Y := Y;
end;

function Vec2f(V: TVec2i): TVec2f; MathDir
begin
  Vec2f := Vec2f(V.X, V.Y);
end;

function Add2f(A, B: TVec2f): TVec2f; MathDir
begin
  Add2f.X := A.X + B.X;
  Add2f.Y := A.Y + B.Y;
end;

function Sub2f(A, B: TVec2f): TVec2f; MathDir
begin
  Sub2f.X := A.X - B.X;
  Sub2f.Y := A.Y - B.Y;
end;

function Mul2f(A: TVec2f; S: single): TVec2f; MathDir
begin
  Mul2f.X := S * A.X;
  Mul2f.Y := S * A.Y;
end;

function Len2f(A: TVec2f): Single; MathDir
begin
  Len2f := Sqrt(Sqr(A.X) + Sqr(A.Y));
end;

function Dot2f(A, B: TVec2f): Single; MathDir
begin
  Dot2f := A.X * B.X  +  A.Y * B.Y;
end;

function Norm2f(V: TVec2f): TVec2f; MathDir
begin
  Norm2f := V / Len2f(V);
end;
//}}}
//{{{ TVec4f
function Vec4f(X, Y, Z, W: Single): TVec4f; MathDir
begin
  Vec4f.X := X;
  Vec4f.Y := Y;
  Vec4f.Z := Z;
  Vec4f.W := W;
end;

function Add4f(A, B: TVec4f): TVec4f; MathDir
begin
  Add4f.X := A.X + B.X;
  Add4f.Y := A.Y + B.Y;
  Add4f.Z := A.Z + B.Z;
  Add4f.W := A.W + B.W;
end;

function Sub4f(A, B: TVec4f): TVec4f; MathDir
begin
  Sub4f.X := A.X - B.X;
  Sub4f.Y := A.Y - B.Y;
  Sub4f.Z := A.Z - B.Z;
  Sub4f.W := A.W - B.W;
end;

function Mul4f(A: TVec4f; S: Single): TVec4f; MathDir
begin
  Mul4f.X := A.X * S;
  Mul4f.Y := A.Y * S;
  Mul4f.Z := A.Z * S;
  Mul4f.W := A.W * S;
end;
//}}}
//{{{ TRect
function Rect(A, B: TVec2f): TRect;
begin
  Rect.A := A;
  Rect.B := B;
end;

function Contain(A, B: TRect): Boolean;
begin
  Contain := (A.A.X <= B.A.X) and (B.B.X <= A.B.X) and
             (A.A.X <= B.A.X) and (B.B.X <= A.B.X);
end;

function Intersect1D(A, B, C, D: Single): Boolean;
begin
  Intersect1D := ((A <= C)and(C <= B)) or ((A <= D)and(D <= B)) or
                 ((C <= A)and(A <= D)) or ((C <= B)and(B <= D));
end;

function Intersect(A, B: TRect): Boolean; overload;
begin
  Intersect := Intersect1D(A.A.X, A.B.X, B.A.X, B.B.X) and
               Intersect1D(A.A.Y, A.B.Y, B.A.Y, B.B.Y);
end;

function Intersect(R: TRect; X1, X2, Y: Single): Boolean;
begin
  Intersect := Intersect(R, Rect(Vec2f(X1, Y), Vec2f(X2, Y)));
end;

function Intersect(R: TRect; A: TVec2f; var Delta: TVec2f): Boolean;
begin
  if A.X > R.B.X then
    Delta.X := A.X - R.B.X
  else if A.X < R.A.X then
    Delta.X := A.X - R.A.X
  else
    Delta.X := 0.0;
  if A.Y > R.B.Y then
    Delta.Y := A.Y - R.B.Y
  else if A.Y < R.A.Y then
    Delta.Y := A.Y - R.A.Y
  else
    Delta.Y := 0.0;
  Intersect := (Delta.X = 0.0) and (Delta.Y = 0.0);
end;

function Intersect(R: TRect; A: TVec2f): Boolean;
begin
  Intersect := (R.A.X <= A.X) and (A.X <= R.B.X) and
               (R.A.Y <= A.Y) and (A.Y <= R.B.Y)
end;
//}}}
// TLine2f//{{{
function Line2f(N: TVec2f; D: Single): TLine2f;
begin
  Line2f.N := N;
  Line2f.D := D;
end;

function Line2f(A, B: TVec2f): TLine2f;
begin
  Line2f.N := (B - A)/Len2f(B - A); 
  Line2f.N := Vec2f(- Line2f.N.Y, Line2f.N.X);
  Line2f.D := - Dot2f(A, Line2f.N);
end;

function Dist2f(Line: TLine2f; P: TVec2f): Single; overload;
begin
  Dist2f := Dot2f(Line.N, P) + Line.D;
end;

function Intersect(Line: TLine2f; Rect: TRect; var T: Single): Boolean;
begin
end;

function Intersect(Pos, Dir: TVec2f; Rect: TRect; var T: Single): Boolean;
  var
    Line: TLine2f;
    V: array[0..3] of record
                     P: TVec2f;
                     D: Single;
                   end;
    I: Integer;
    Temp, A, B: Single;
    P: TVec2f;
begin
  Intersect := False;
  T := 10e+10;
  V[0].P := Rect.A;
  V[1].P := Vec2f(Rect.B.X, Rect.A.Y);
  V[2].P := Rect.B;
  V[3].P := Vec2f(Rect.A.X, Rect.B.Y);
  Line := Line2f(Pos, Pos + Dir);
  Dir := Dir / Len2f(Dir);
  for I := 0 to 3 do
    V[I].D := Dist2f(Line, V[I].P);
  for I := 0 to 3 do 
    if V[I].D * V[(I+1) mod 4].D < 0 then begin
      A := Abs(V[I].D);
      B := Abs(V[(I+1) mod 4].D);
      P := (B * V[I].P  +  A * V[(I + 1) mod 4].P) / (A + B);
      Temp := Dot2f(Dir, P - Pos);
      if (0 < Temp) and (Temp < T) then begin
        T := Temp;
        Intersect := True;
      end;
    end;
end;
//}}}
// operators//{{{
{$IFDEF OPMATH}
operator = (A, B: TVec2i) Result: Boolean; inline;
begin
  Result := (A.X = B.X) and (A.Y = B.Y);
end;

operator := (A: TVec2i) Result: TVec2f; inline;
begin
  Result := Vec2f(A.X, A.Y)
end;

operator + (A, B: TVec2i) Result: TVec2i; inline;
begin
  Result := Add2i(A, B);
end;

operator + (A, B: TVec2f) Result: TVec2f; inline;
begin
  Result := Add2f(A, B);
end;

operator + (A, B: TVec4f) Result: TVec4f; inline;
begin
  Result := Add4f(A, B);
end;

operator - (A, B: TVec2i) Result: TVec2i; inline;
begin
  Result := Sub2i(A, B);
end;

operator - (A, B: TVec2f) Result: TVec2f; inline;
begin
  Result := Sub2f(A, B);
end;

operator - (A, B: TVec4f) Result: TVec4f; inline;
begin
  Result := Sub4f(A, B);
end;

operator * (I: integer; A: TVec2i) Result: TVec2i; inline;
begin
  Result := Mul2i(A, I)
end;

operator * (A: TVec2i; I: Integer) Result: TVec2i; inline;
begin
  Result := Mul2i(A, I)
end;

operator * (S: Single; A: TVec2f) Result: TVec2f; inline;
begin
  Result := Mul2f(A, S)
end;

operator * (A: TVec2f; S: Single) Result: TVec2f; inline;
begin
  Result := Mul2f(A, S)
end;

operator * (A: TVec4f; S: Single) Result: TVec4f; inline;
begin
  Result := Mul4f(A, S);
end;

operator * (S: Single; A: TVec4f) Result: TVec4f; inline;
begin
  Result := Mul4f(A, S);
end;

operator div (A: TVec2i; I: Integer) Result: TVec2i; inline;
begin
  Result := Vec2i(A.X div I, A.Y div I);
end;

operator / (A: TVec2f; S: Single) Result: TVec2f; inline;
begin
  Result := Mul2f(A, 1/S);
end;
{$ENDIF OPMATH}
//}}}

end.
