unit DGuiCommon;

interface

uses
  DMath,
  DGuiCore;

type
TLabel = class(TWidgetDecorator)
private
  FColor: TVec4f;
  FFont: TGuiFont;
  FText: String;
  FAlign: TVec2f;
protected
  function GetFontName: String;
  procedure SetFont(NewFont: String); virtual;
  procedure SetText(NewText: String); virtual;
  procedure SetAlign(NewAlign: TVec2f); virtual;
public
  constructor Create(Widget: TWidget);
  procedure DrawLabel;
  function GetTextRect: TRect;
  property Color: TVec4f read FColor write FColor;
  property Font: String read GetFontName write SetFont;
  property Text: String read FText write SetText;
  property Align: TVec2f read FAlign write SetAlign;
end;

TGuiLabel = class(TWidget)
private
  FLabel: TLabel;
  procedure SetColor(NewColor: TVec4f); inline;
  function GetColor: TVec4f; inline;
  procedure SetFont(NewFont: String); inline;
  function GetFont: String; inline;
  procedure SetAlign(NewAlign: TVec2f); inline;
  function GetAlign: TVec2f; inline;
public
  constructor Create(Gui: TGui; Name, Style: String);
  destructor Destroy; override;
  procedure Render; override;
  property _Label: TLabel read FLabel;
  property Color: TVec4f read GetColor write SetColor;
  property Font: String read GetFont write SetFont;
  property Align: TVec2f read GetAlign write SetAlign;
end;

TGuiButton = class(TWidget)
  procedure Render; override;
end;

implementation

constructor TLabel.Create(Widget: TWidget);
begin
  inherited Create(Widget);
  FFont := nil;
  FText := '';
  FAlign := Vec2f(0.5, 0.5);
  FColor := Widget.Style.PrimaryColor;
end;

procedure TLabel.DrawLabel;
begin  
  Style.DrawText(Widget.GlobalCenter, FAlign, FFont, FText);
end;

function TLabel.GetTextRect;
begin
  Result.A := Widget.GlobalCenter - FAlign * Vec2f(FFont.GetWidth(FText),
                                                   FFont.GetHeight(FText));
  Result.B := Result.A + Vec2f(FFont.GetWidth(FText),
                               FFont.GetHeight(FText));
end;

function TLabel.GetFontName: String;
begin
  Result := '<undefined>';
  if FFont <> nil then
    Result := FFont.Name;
end;

procedure TLabel.SetFont(NewFont: String);
begin
  FFont := Widget.Style.FindFont(NewFont);
end;

procedure TLabel.SetText(NewText: String);
begin
  FText := NewText;
end;

procedure TLabel.SetAlign(NewAlign: TVec2f);
begin
  FAlign := NewAlign;
end;

procedure TGuiLabel.SetColor;
begin
  FLabel.Color := NewColor;
end;

function TGuiLabel.GetColor;
begin
  Result := FLabel.Color;
end;

procedure TGuiLabel.SetFont;
begin
  FLabel.Font := NewFont;
end;

function TGuiLabel.GetFont;
begin
  Result := FLabel.Font;
end;

procedure TGuiLabel.SetAlign;
begin
  FLabel.Align := NewAlign;
end;

function TGuiLabel.GetAlign;
begin
  Result := FLabel.Align;
end;

constructor TGuiLabel.Create(Gui: TGui; Name, Style: String);
begin
  inherited Create(Gui, Name, Style);
  FLabel := TLabel.Create(Self);
  FLabel.Color := Self.Style.PrimaryColor;
end;

destructor TGuiLabel.Destroy;
begin
  FLabel.Free;
  inherited Destroy;
end;

procedure TGuiLabel.Render;
begin
  Style.Color4f(Color);
  FLabel.DrawLabel;
end;

procedure TGuiButton.Render;
begin
  inherited Render;
  Style.DrawRect(GlobalRect);
end;

end.
