unit DAI;

interface

uses
  DUtils,
  DMath;

type
TAICell = record
  Pos: TVec2i;
  Last: Boolean;
  Power: Integer; 
end;

TAI = class
private
  FPlayer: Integer;
  FThinkState: Integer;
  FStartThink: Integer;
  FThinkCommand: Integer;
  FStartWait: Integer;
  FThinkTime: Integer;

  //  
  FState: Integer; // 0   1  2  
  FDebutTurns: Integer;
  FDebut: Integer;
  FLast: TVec2i;
  FPossibles: array of TAICell; //  

  // 
  FCriticalSection: TCriticalSection;
  FTurns: Array Of TVec2i;
  FReaded: Boolean;
  
  function GetThinkString: String;
  // multithread functions
  function Readed: Boolean;
  function SimpleEnemy(X, Y: Integer): Boolean;
  function FindTurn: TVec2i;
  procedure AddTurn(V: TVec2i);
  procedure AddPossible(V: TVec2i);
  procedure BuildPossibles;
  procedure Think;
public
  constructor Create(Player: Integer);
  procedure Update;
  property Player: Integer read FPlayer;
  property ThinkState: Integer read FThinkState;
  property ThinkString: String read GetThinkString;
  property ThinkTime: Integer read FThinkTime write FThinkTime;
end;

implementation

uses
  DGame;

function ThreadThink(Param: Pointer): PtrInt;
  var
    AI: TAI absolute Param;
begin
  AI.Think;
end;

function TAI.Readed: Boolean;
begin
  FCriticalSection.Enter;
  Result := FReaded;
  FCriticalSection.Leave;
end;

function TAI.SimpleEnemy(X, Y: Integer): Boolean;
begin
  if not Game.InMap(Vec2i(X, Y)) then
    Result := False
  else
    with Game.Cells[Vec2i(X, Y)] do
      Result := Bug and (FPlayer <> Player) and (State = bsBug);
end;

function TAI.FindTurn: TVec2i;
  var
    I, J: Integer;
    Up: Integer;
    Turns: Array Of Integer;
  function GetDepth(P: TVec2i): Integer;
  begin
    Result := 1000;
    if Game.Cells[P + Vec2i( +1,  0)].Bug and (Game.Cells[P + Vec2i( +1,  0)].Player = Player) then
      Result := Min(Game.Cells[P + Vec2i( +1,  0)].Depth, Result);
    if Game.Cells[P + Vec2i( -1,  0)].Bug and (Game.Cells[P + Vec2i( -1,  0)].Player = Player) then
      Result := Min(Game.Cells[P + Vec2i( -1,  0)].Depth, Result);
    if Game.Cells[P + Vec2i(  0, +1)].Bug and (Game.Cells[P + Vec2i(  0, +1)].Player = Player) then
      Result := Min(Game.Cells[P + Vec2i(  0, +1)].Depth, Result);
    if Game.Cells[P + Vec2i(  0, -1)].Bug and (Game.Cells[P + Vec2i(  0, -1)].Player = Player) then
      Result := Min(Game.Cells[P + Vec2i(  0, -1)].Depth, Result);
    if Game.Cells[P + Vec2i( -1, -1)].Bug and (Game.Cells[P + Vec2i( -1, -1)].Player = Player) then
      Result := Min(Game.Cells[P + Vec2i( -1, -1)].Depth, Result);
    if Game.Cells[P + Vec2i( +1, -1)].Bug and (Game.Cells[P + Vec2i( +1, -1)].Player = Player) then
      Result := Min(Game.Cells[P + Vec2i( +1, -1)].Depth, Result);
    if Game.Cells[P + Vec2i( +1, +1)].Bug and (Game.Cells[P + Vec2i( +1, +1)].Player = Player) then
      Result := Min(Game.Cells[P + Vec2i( +1, +1)].Depth, Result);
    if Game.Cells[P + Vec2i( -1,  +1)].Bug and (Game.Cells[P + Vec2i( -1, +1)].Player = Player) then
      Result := Min(Game.Cells[P + Vec2i( -1, +1)].Depth, Result);
  end;
begin
  BuildPossibles;
  if Length(FPossibles) = 0 then
    Exit; //  :(
  if FState = 0 then begin
    //   ,        
    Up := 0;
    for I := 0 to 7 do
      for J := 0 to 7 do
        if Game.Cells[Vec2i(I, J)].Bug and 
           (Game.Cells[Vec2i(I, J)].Player <> FPlayer) then
          if I > J then 
            Inc(Up)
          else if I < J then
            Dec(Up);
    if Up > 0 then
      FDebut := 0
    else if Up < 0 then
      FDebut := 1
    else
      FDebut := Random(2);
    FState := 1;
  end;
  // 1.     
  for I := 0 to High(FPossibles) do
    if Game.Cells[FPossibles[I].Pos].Bug and
       (Game.Cells[FPossibles[I].Pos].Player <> FPlayer) then begin
      //      -  
      FPossibles[I].Power := 1000 - Game.Cells[FPossibles[I].Pos].Depth;
      if Game.Cells[FPossibles[I].Pos].Connected then
        Inc(FPossibles[I].Power, 1000);
    end else
      with FPossibles[I], FPossibles[I].Pos do begin
        Power := 0;
        if SimpleEnemy(X-1, Y-1) then
          Power := Max(Power, 500 - Game.Cells[Vec2i(X-1, Y-1)].Depth);
        if SimpleEnemy(X+1, Y-1) then
          Power := Max(Power, 500 - Game.Cells[Vec2i(X+1, Y-1)].Depth);
        if SimpleEnemy(X+1, Y+1) then
          Power := Max(Power, 500 - Game.Cells[Vec2i(X+1, Y+1)].Depth);
        if SimpleEnemy(X-1, Y+1) then
          Power := Max(Power, 500 - Game.Cells[Vec2i(X-1, Y+1)].Depth);
        if SimpleEnemy(X-1, Y+0) then
          Power := Max(Power, 500 - Game.Cells[Vec2i(X-1, Y+0)].Depth);
        if SimpleEnemy(X+1, Y+0) then
          Power := Max(Power, 500 - Game.Cells[Vec2i(X+1, Y+0)].Depth);
        if SimpleEnemy(X+0, Y-1) then
          Power := Max(Power, 500 - Game.Cells[Vec2i(X+0, Y-1) ].Depth);
        if SimpleEnemy(X+0, Y+1) then
          Power := Max(Power, 500 - Game.Cells[Vec2i(X+0, Y+1)].Depth);
      end;
  {for I := 0 to High(FPossibles) do
    if Game.Cells[FPossibles[I].Pos].Bug and 
       (Game.Cells[FPossibles[I].Pos].Player <> FPlayer) then begin
      AddTurn(FPossibles[I].Pos);
      goto Next;
    end;}
  // 2.    
  SetLength(Turns, 1);
  Turns[0] := 0;
  for I := 0 to High(FPossibles) do
    if FPossibles[I].Power > FPossibles[Turns[0]].Power then begin
      SetLength(Turns, 1);
      Turns[0] := I;
    end else if FPossibles[I].Power = FPossibles[Turns[0]].Power then begin         
      SetLength(Turns, Length(Turns) + 1);
      Turns[High(Turns)] := I;
    end;
  if FPossibles[Turns[0]].Power = 0 then begin
    SetLength(Turns, 0);
    for I := 0 to High(FPossibles) do
      FPossibles[I].Power := 100*Ord(Ord(I < J) = FDebut) + 100*Ord(Ord(I > J) = FDebut) + GetDepth(FPossibles[I].Pos);
    SetLength(Turns, 1);
    Turns[0] := 0;
    for I := 0 to High(FPossibles) do
      if FPossibles[I].Power > FPossibles[Turns[0]].Power then begin
        SetLength(Turns, 1);
        Turns[0] := I;
      end else if FPossibles[I].Power = FPossibles[Turns[0]].Power then begin         
        SetLength(Turns, Length(Turns) + 1);
        Turns[High(Turns)] := I;
      end;
  end; 
  Result := FPossibles[Turns[Random(Length(Turns))]].Pos;
end;

procedure TAI.AddTurn(V: TVec2i);
begin
  FCriticalSection.Enter;
  SetLength(FTurns, Length(FTurns) + 1);
  FTurns[High(FTurns)] := V;
  FReaded := False;
  FCriticalSection.Leave;
end;

procedure TAI.AddPossible(V: TVec2i);
begin
  SetLength(FPossibles, Length(FPossibles) + 1);
  FPossibles[High(FPossibles)].Pos := V;
end;

procedure TAI.BuildPossibles;
  var
    I, J: Integer;
begin
  SetLength(FPossibles, 0);
  for I := 0 to 7 do
    for J := 0 to 7 do
      if Game.Cells[Vec2i(I, J)].CanPut then
        AddPossible(Vec2i(I, J));
end;

procedure TAI.Think;
  label
    Start,
    Next,
    MakeOff;
  var
    StartWait, Count: Integer;
    I: Integer;
begin
  Count := Game.BugsCount;
  FCriticalSection.Enter;
  SetLength(FPossibles, 0);
  FReaded := True;
  FCriticalSection.Leave;
Start:
  //FCriticalSection.Enter;
    while not Readed do;
  //FCriticalSection.Leave;
  AddTurn(FindTurn);
Next:
  StartWait := Game.Time;
  while Game.Time - StartWait < 200 do;
  Dec(Count);
  if Count > 0 then
    goto Start;
  while not Readed do;
  Count := Game.BugsCount;
  if (FPlayer = Game.CurPlayer) and (Count > 0) then
    goto Start;
MakeOff:
    Exit;
end;

function TAI.GetThinkString: String;
begin
  case FThinkState of
    0: Result := '';
    1: Result := '';
    2: Result := '';
    3: Result := '';
    4: Result := '';
  end;
end;

constructor TAI.Create(Player: Integer);
begin
  FPlayer := Player;
  FThinkState := 0;
  FState := 0;
  FCriticalSection := TCriticalSection.Create;
end;

procedure TAI.Update;
  var
    I: Integer;
begin
  if Game.CurPlayer = FPlayer then
    if FThinkState < 3 then begin
      FThinkState := 3;
      SetLength(FTurns, 0);
      FStartWait := Game.Time;
      //BeginThread(ThreadThink, Self);
    end else begin
      if Game.Time - FStartWait < FThinkTime then
        Exit;
      Game.Put(FindTurn);
      FStartWait := Game.Time;
      {FCriticalSection.Enter;
      if Length(FTurns) > 0 then begin
        for I := 0 to High(FTurns) do
          Game.Put(FTurns[I]);
      end; 
      SetLength(FTurns, 0);
      FReaded := True;
      FCriticalSection.Leave;}   
    end
  else begin
    if FThinkState >= 3 then
      FThinkState := 0;
  end;
end;

end.
