Unit DCore;

interface

Uses
  DDebug,
  DUtils,
  DMeta;

Type
TEvent = class;
TEntity = class;
TClassEntity = class of TEntity;
TList = class;
TDecorator = class;
TDecoratable = class;

TCompareFunc = Function(A, B: TEntity): Integer;

TEvent = class//{{{
private
  FSender: TEntity;
  FID: String;
  FDone: Boolean;
public
  Constructor Create(Sender: TEntity); 
  Function IsType(ID: String): Boolean; virtual;
  Function IsSubType(ID: String): Boolean;
  Property Sender: TEntity read FSender;
  Property ID: String read FID;
  Property Done: Boolean read FDone write FDone;
end;
//}}}
TEvCreated = class(TEvent)//{{{
private
  FEntity: TEntity;
public
  Constructor Create(Sender: TEntity; Entity: TEntity);
  Property Entity: TEntity read FEntity;
end;
//}}}
TEvRender = class(TEvent)//{{{
end;
//}}}
TEvUpdate = class(TEvent)//{{{
end;
//}}}
TEvDestroying = class(TEvCreated)//{{{
end;
//}}}
TEvMessage = class(TEvent)//{{{
private
  FMessage: String;
public
  Constructor Create(Sender: TEntity; Message: String); 
  Function IsType(ID: String): Boolean; override;
  Property Message: String read FMessage;  
end;
//}}}
//  TEntity//{{{
//            
//
//        :
//        
TEntity = class
private
  FOwner: TEntity;
  FGrabber: Boolean;
  FHash: Cardinal;
protected
  FName: String;
public
  Constructor Create(Owner: TEntity); virtual;
  Destructor Destroy; override;
  Procedure DoString(S: String); virtual;
  Procedure AssignVar(Name, Value: String); virtual;
  Procedure Render; virtual;
  Procedure Update; virtual;
  Procedure Kill; virtual;
  Procedure OnEvent(Event: TEvent); virtual;
  Procedure SendEvent(Event: TEvent); virtual;
  Procedure SendEventAndFree(Event: TEvent);
  Property Grabber: Boolean read FGrabber;
  Property Owner: TEntity read FOwner; 
  Property Hash: Cardinal read FHash;
  Property Name: String read FName;
end;
//}}}
TList = class(TEntity)//{{{
private
  FEntities: Array Of TEntity;
public
  Procedure Render; override;
  Procedure  Update; override;
  Procedure SendEvent(Event: TEvent); override;
  Procedure GrabberCollect;
  Procedure Add(TheObject: TEntity);
  Procedure Del(TheObject: TEntity);
  Function Find(Entity: TEntity): Integer;
  Procedure AddExclusive(TheObject: TEntity);
  Procedure FreeAll;
  Function Last: TEntity;
  Procedure Union(A, B: TList);
  Procedure AddList(Other: TList);
  Procedure Sort(CompareFunc: TCompareFunc);
  Function GetEntity(Index: Integer): TEntity;
  Procedure SetEntity(Index: Integer; Entity: TEntity);
  Function Length: Integer;
  Function High: Integer;
  Procedure SetLength(Len: Integer);
  Property Entities[Index: Integer]: TEntity read GetEntity 
                                             write SetEntity; default;
end;
//}}}
TNamedList = class(TEntity)//{{{
private
  FNames: Array Of String;
  FEntities: TList;
  Procedure Add(Name: String; Entity: TEntity);
public
  Constructor Create(Owner: TEntity); override;
  Destructor Destroy; override;
  Procedure SendEvent(Event: TEvent); override;
  Procedure FreeAll;
  Function GetEntity(Name: String): TEntity;
  Procedure SetEntity(Name: String; Entity: TEntity);
  Function GetName(I: Integer): String;
  Property Items[Index: String]: TEntity read GetEntity 
                                         write SetEntity; default;
  Property List: TList read FEntities;
end;
//}}}
//      TListOfString//{{{
//      ,    
//
//        [] ,     
//      ,  ''
//        [] ,    0, 
//        ,    High,     
//      
//      
//      ReadFromTextFile -    
TListOfString = class(TEntity)
private
  FStrings: Array Of String;
public
  Constructor Create(Owner: TEntity); override;
  Procedure ReadFromTextFile(TextData: TTextData);
  Function Length: Integer;
  Function High: Integer;
  Procedure SetLength(Len: Integer);
  Function GetBottom(ID: Integer): String;
  Procedure Add(S: String);
  Function GetString(ID: Integer): String;
  Procedure SetString(ID: Integer; S: String);
  Property Strings[ID: Integer]: String read GetString write SetString; default;
end;
//}}}
TTypeClass = class(TEntity)//{{{
protected
  FValue: Pointer;
  FNeedFree: Boolean;
public
  Function ToString: String; virtual;
  Function ToInteger: Integer; virtual;
  Function ToDouble: Double; virtual;
  Procedure SetString(S: String); virtual;
  Procedure SetInteger(I: Integer); virtual;
  Procedure SetDouble(D: Double); virtual;
end;
//}}}
//      TVarList//{{{
//       
// 
//        RegistVar -       V
//        Unregist -   (   
//                       )
//                        TTypeClass',
//                        
//                   .
//        GetVar -     
TVarList = class(TEntity)
private
  FVars: TNamedList;
public
  Constructor Create(Owner: TEntity); override;
  Destructor Destroy; override;
  Procedure RegistVar(Name: String; V: TTypeClass);
  Procedure Unregist(V: TTypeClass); overload;
  Procedure Unregist(Name: String); overload;
  Procedure UnregistAndFree(V: TTypeClass); overload;
  Procedure UnregistAndFree(Name: String); overload;
  Function GetVar(Name: String): TTypeClass;
  Function VarCount: Integer;
  Function GetVarName(I: Integer): String;
  Function GetVarValue(I: Integer): TTypeClass;
  Property Names[Index: Integer]: String read GetVarName;
  Property Values[Index: Integer]: TTypeClass read GetVarValue;
  Property VarList: TNamedList read FVars;
  Property Vars[Index: String]: TTypeClass read GetVar; default; 
end;
//}}}
//      TDecorator//{{{
//       - ,     
//
//           Owner   -
//
//        GetDecoratable -  - (=TDecoratable(Owner))
//        GetVar('...') -    
//                              (=Decoaratable.GetSharedVar)
TDecorator = class(TEntity)
  Function GetDecoratable: TDecoratable;
  Function GetVar(Name: String): TMetaVar;
  property Vars[Name: string]: TMetaVar read GetVar;
  Property Decoratable: TDecoratable read GetDecoratable;
end;
//}}}
//       - ,     .//{{{
//
//      FDecoratbles - ,    
//      FSharedVasr -    
//
//      RegistSharedVar -   
//      GetVar -      ,
//                ,   TTypeClass
//      SendEvent -     
//      HasDecoratot -     
//      FindDecorator -     ,
//                  Nil
//      Decorators -    
TDecoratable = class(TEntity)
private
  FDecorators: TList;
  FSharedVars: TMetaTable;
public
  Constructor Create(Owner: TEntity); override;
  Destructor Destroy; override;
  Procedure AddDecorator(Decorator: TDecorator);
  //Procedure RegistSharedVar(Name: String; V: TTypeClass);
  Function GetVar(Name: String): TMetaVar; inline;
  Procedure SendEvent(Event: TEvent); override;
  Function HasDecorator(DecType: TClassEntity): Boolean; overload;
  Function HasDecorator(DecType: String): Boolean; overload;
  Function FindDecorator(DecType: String): TDecorator;
  property Vars[Name: string]: TMetaVar read GetVar;
  Property Decorators: TList read FDecorators;
end;
//}}}
//      TFactory//{{{
//       - ,      
//         
//
//      RegistClass -   TypeName
//      CreateEntity -     TypeName 
//         Owner
TFactory = class(TObject)
private
  FClasses: Array Of TClassEntity;
public
  Procedure RegistClass(TheClass: TClassEntity);
  Function CreateEntity(TypeName: String; Owner: TEntity): TEntity;
end;
//}}}
// TEventQueItem//{{{
PEventQueItem = ^TEventQueItem;
TEventQueItem = record
  Event: TEvent;
  Next: PEventQueItem;
  //  Event = nil,    
end;
//}}}
// TEventQue -      //{{{
TEventQue = class(TEntity)
private
  FCritical: TRTLCriticalSection; //   
  FLast: PEventQueItem;
  FHead: PEventQueItem;
public
  constructor Create(Owner: TEntity); override;
  destructor Destroy; override;
  procedure ClearAll;
  procedure Que(Event: TEvent);
  function EnQue: TEvent; overload;
  function EnQue(var Event: TEvent): Boolean; overload;
  function EnQueAll: PEventQueItem;
end;
//}}}

Var
  //    
  Factory: TFactory;
  // ,    
  NullTypeClass: TTypeClass;

implementation

Uses
  DParser,
  DTypeClasses;

//  TEvent//{{{
//
Constructor TEvent.Create;
begin
  Inherited Create;
  FSender := Sender;
  FID := AnsiLowerCase(ClassName);
  FDone := False;
end;

Function TEvent.IsType(ID: String): Boolean;
begin
  Result := FID = AnsiLowerCase(ID);
end;

Function TEvent.IsSubType(ID: String): Boolean;
  Var
    S: String;
begin
  S := FID;
    If Length(ID) < Length(FID) then
      SetLength(S, Length(ID));
  Result := AnsiLowerCase(ID) = S;
end;
//}}}
//  TEvCreated//{{{
//
Constructor TEvCreated.Create(Sender: TEntity; Entity: TEntity);
begin
  Inherited Create(Sender);
  FEntity := Entity;
end;
//}}}
//  TEvMessage//{{{
//
Constructor TEvMessage.Create;
begin
  Inherited Create(Sender);
  FMessage := AnsiLowerCase(Message);
end;

Function TEvMessage.IsType(ID: String): Boolean;
begin
  Result := (Inherited IsType(ID))Or(AnsiLowerCase(ID) = FMessage);
end;
//}}}
//  TEntity//{{{
//
Constructor TEntity.Create(Owner: TEntity);
begin
  Inherited Create;
  FOwner := Owner;
  FGrabber := False;
  FHash := Random(4096);
end;

Destructor TEntity.Destroy;
begin
  //Log.PrintLnSingleton('Destroyed: ' + ClassName);
  Inherited Destroy;
end;

Procedure TEntity.DoString(S: String);
  Var
    I: Integer;
    List: TArrayOfString;
    Name, Value: String;
    C: Char;
begin
  ParseList(S, '/', List);
    For I := 0 to High(List) do begin
        //If (Pos('=', List[I]) <> 0)And
        //   (Pos('=', List[I]) < Pos(':', List[I])) then
          C := '=';
        //else
        //  C := ':';
      Name := StringDeletePost(List[I], Pos(C, List[I])); 
      Value := StringDeletePre(List[I], Pos(C, List[I]));
      AssignVar(Name, Value);
    end;
end;

Procedure TEntity.AssignVar(Name, Value: String);
begin
    If Name = 'name' then
      FName := Value;
end;

Procedure TEntity.Render;
begin
end;

Procedure TEntity.Update;
begin
end;

Procedure TEntity.Kill;
begin
  FGrabber := True;
end;

Procedure TEntity.OnEvent(Event: TEvent);
begin
    If Event.IsType('TEvRender') then
      Render
    else If Event.IsType('TEvUpdate') then
      Update;         
end;

Procedure TEntity.SendEvent(Event: TEvent); 
begin
  OnEvent(Event);
end;

Procedure TEntity.SendEventAndFree(Event: TEvent);
begin
  SendEvent(Event);
  Event.Free;
end;
//}}}
//  TList//{{{
//
Procedure TList.Render;
  Var
    I: Integer;
begin
    For I := 0 to High do
      FEntities[I].Render;
end;

Procedure TList.Update;
  Var
    I: Integer;
begin
    For I := 0 to High do
      FEntities[I].Update;
end;

Procedure TList.SendEvent(Event: TEvent);
  Var
    I: Integer;
begin
    For I := 0 to High do
      FEntities[I].SendEvent(Event);
end;

Procedure TList.GrabberCollect;
  Var
    I: Integer;
begin{
  I := High;
    While I >= 0 do begin
        If FEntities[I].Grabber then begin
          SendEventAndFree(TEvDestroying.Create(Self, FEntities[I]));
          FEntities[I].Free;
          FEntities[I] := Nil;
          FEntities[I] := FEntities[High];
          SetLength(Length-1);
        end;
      Dec(I);
    end;}
    For I := High downto 0 do begin
      If FEntities[I].Grabber then begin
        FEntities[I].Free;
        FEntities[I] := Nil;
        SetLength(Length-1);
      end;
    end;
  Del(Nil);
  {
  L := 0;
    For I := 0 to High do
      If FEntities[I] <> Nil then begin
        FEntities[L] := FEntities[I];
        Inc(L);
      end;
  SetLength(L);
    For I := 0 to High do
      Assert(FEntities[I] <> Nil);}
end;

Procedure TList.Add(TheObject: TEntity);
begin
  SetLength(Length + 1);
  FEntities[High] := TheObject;
end;

Procedure TList.Del(TheObject: TEntity);
  Var
    I, Left: Integer;
begin
  I := 0;
    While I < Length do begin
        If FEntities[I] = TheObject then begin
          FEntities[I] := Nil;
          //FEntities[I] := Last;
          //SetLength(Length - 1);
        end;
      Inc(I);
    end;
  I := 0;
  Left := 0;
  while I < Length do begin
    if FEntities[I] <> Nil then begin
      FEntities[Left] := FEntities[I];
        if Left <> I then
          FEntities[I] := Nil;
      Inc(Left);
    end;
    Inc(I);
  end;
  SetLength(Left);
end;

Function TList.Find(Entity: TEntity): Integer;
  Var
    I: Integer;
begin
    For I := 0 to High do
      If FEntities[I] = Entity then begin
        Result := I;
        Exit;
      end;
  Result := -1;
end;

Procedure TList.AddExclusive(TheObject: TEntity);
begin
    If Find(TheObject) = -1 then
      Add(TheObject);
end;

Procedure TList.FreeAll;
  Var
    I: Integer;
begin
    For I := 0 to High do
      FEntities[I].Free;
  SetLength(0);
end;

Function TList.Last: TEntity;
begin
  Result := FEntities[High];
end;

Procedure TList.Union(A, B: TList);
  Var
    I: Integer;
begin
  SetLength(0);
    For I := 0 to A.High do
      Add(A[I]);
    For I := 0 to B.High do
      Add(B[I]);
end;

Procedure TList.AddList(Other: TList);
  Var
    I: Integer;
begin
    For I := 0 to Other.High do
      Add(Other[I]);
end;

Procedure TList.Sort(CompareFunc: TCompareFunc); 
  Procedure QSort(A, B: Integer);
    Var
      M, L, R: Integer;
      Tmp: TEntity;
  begin
    L := A;
    R := B;
    M := (A + B) div 2;
      While L < R do begin
          While (L < M)And(CompareFunc(FEntities[L], FEntities[M]) = -1) do
            Inc(L);
          While (M < R)And(CompareFunc(FEntities[M], FEntities[R]) = -1) do
            Dec(R);
        Tmp := FEntities[L];
        FEntities[L] := FEntities[R];
        FEntities[R] := Tmp;
          If (L = M) then
            M := R
          else If (R = M) then
            M := L;
        Inc(L);
        Dec(R);
      end;
  end;
begin
  QSort(0, High);
end;

Function TList.GetEntity(Index: Integer): TEntity;
begin
  Result := FEntities[Index];
end;

Procedure TList.SetEntity(Index: Integer; Entity: TEntity);
begin
  FEntities[Index] := Entity;
end;

Function TList.Length: Integer;
begin
  Result := System.Length(FEntities);
end;

Function TList.High: Integer;
begin
  Result := System.High(FEntities);
end;

Procedure TList.SetLength(Len: Integer);
begin
  System.SetLength(FEntities, Len);
end;
//}}}
//  TNamedList//{{{
//
Procedure TNamedList.Add(Name: String; Entity: TEntity);
begin
  SetLength(FNames, Length(FNames) + 1);
  FNames[High(FNames)] := Name;
  FEntities.Add(Entity);
end;

Constructor TNamedList.Create;
begin
  Inherited Create(Owner);
  SetLength(FNames, 0);
  FEntities := TList.Create(Self);
end;

Destructor TNamedList.Destroy;
begin
  FEntities.Free;
  Inherited Destroy;
end;

Procedure TNamedList.SendEvent(Event: TEvent);
begin
  Inherited SendEvent(Event);
  FEntities.SendEvent(Event);
end;

Procedure TNamedList.FreeAll;
begin
  FEntities.FreeAll;
  SetLength(FNames, 0);
  FEntities.SetLength(0);
end;

Function TNamedList.GetEntity(Name: String): TEntity;
  Var
    I: Integer;
begin
    For I := 0 to High(FNames) do
      If FNames[I] = Name then begin
        Result := FEntities[I];
        Exit;
      end;
  Result := Nil;
end;

Procedure TNamedList.SetEntity(Name: String; Entity: TEntity);
  Var
    I: Integer;
begin
    For I := 0 to High(FNames) do
      If FNames[I] = Name then begin
        FEntities[I] := Entity;
        Exit;
      end;
  Add(Name, Entity);
end;

Function TNamedList.GetName(I: Integer): String;
begin
  Result := FNames[I];
end;
//}}}
//  TListOfString//{{{
//
Constructor TListOfString.Create(Owner: TEntity);
begin
  Inherited Create(Owner);
  SetLength(0);
end;

Procedure TListOfString.ReadFromTextFile;
  Var
    I: Integer;
begin
  SetLength(TextData.LinesCount);
    For I := 0 to TextData.LinesCount - 1 do
      FStrings[I] := TextData.GetLine(I);
end;

Function TListOfString.Length: Integer;
begin
  Result := System.Length(FStrings);
end;

Function TListOfString.High: Integer;
begin
  Result := System.High(FStrings);
end;

Procedure TListOfString.SetLength(Len: Integer);
begin
  System.SetLength(FStrings, Len);
end;

Function TListOfString.GetBottom(ID: Integer): String;
begin
    If (High - ID < 0)Or(ID < 0) then begin
      Result := '';
      Exit;
    end;
  Result := FStrings[High - ID];
end;

Procedure TListOfString.Add(S: String);
begin
  SetLength(Length + 1);
  FStrings[High] := S;
end;

Function TListOfString.GetString;
begin
    If (ID < 0)Or(ID > High) then
      Result := ''
    else
      Result := FStrings[ID];
end;

Procedure TListOfString.SetString;
begin
  if ID > High then
    SetLength(ID + 1);
  FStrings[ID] := S;
end;
//}}}
//  TTypeClass//{{{
//
Function TTypeClass.ToString;
begin
  Result := '';
end;

Function TTypeClass.ToInteger;
begin
  Result := 0;
end;

Function TTypeClass.ToDouble;
begin
  Result := 0.0;
end;

Procedure TTypeClass.SetString;
begin
end;

Procedure TTypeClass.SetInteger;
begin
end;

Procedure TTypeClass.SetDouble;
begin
end;
//}}}
//  TVarList//{{{
//
Constructor TVarList.Create(Owner: TEntity);
begin
  Inherited Create(Self);
  FVars := TNamedList.Create(Self);
end;

Destructor TVarList.Destroy;
begin
  FVars.FreeAll;
  FVars.Free;
  Inherited Destroy;
end;

Procedure TVarList.RegistVar(Name: String; V: TTypeClass);
begin
  FVars[Name] := V;
end;

Procedure TVarList.Unregist(V: TTypeClass);
begin
  FVars.List.Del(V);
end;

Procedure TVarList.Unregist(Name: String);
begin
  Unregist(GetVar(Name));
end;

Procedure TVarList.UnregistAndFree(V: TTypeClass); overload;
begin
    If (V = NullTypeClass)Or(V = Nil) then
      Exit;
  Unregist(V);
  V.Free;
end;

Procedure TVarList.UnregistAndFree(Name: String); overload;
begin
  UnregistAndFree(GetVar(Name));
end;

Function TVarList.GetVar(Name: String): TTypeClass;
begin
  Result := TTypeClass(FVars[Name]);
    If Result = Nil then begin
      Result := TUniVar.Create(Self);
      RegistVar(Name, Result);
    end;
end;

Function TVarList.VarCount: Integer;
begin
  Result := FVars.List.Length;
end;

Function TVarList.GetVarName(I: Integer): String;
begin
  Result := FVars.GetName(I);
end;

Function TVarList.GetVarValue(I: Integer): TTypeClass;
begin
  Result := TTypeClass(FVars.List[I]);
end;

//}}}
//  TDecorator//{{{
//
Function TDecorator.GetDecoratable: TDecoratable;
begin
  Result := TDecoratable(Owner);
end;

Function TDecorator.GetVar;
begin
  Result := Decoratable.GetVar(Name);
end;
//}}}
//  TDecoratable//{{{
//
Constructor TDecoratable.Create(Owner: TEntity);
begin 
  Inherited Create(Owner);
  FDecorators := TList.Create(Self);
  FSharedVars := TMetaTable.Create;//TVarList.Create(Self);
end;

Destructor TDecoratable.Destroy; 
  Var
    I: Integer;
begin
    For I := 0 to FDecorators.High do begin
        If Owner <> Nil then begin
	  Owner.SendEventAndFree(TEvDestroying.Create(Self, FDecorators[I]))
        end else
          SendEventAndFree(TEvDestroying.Create(Self, FDecorators[I]));
    end;
  FDecorators.FreeAll;
  FDecorators.Free;
  FSharedVars.Free;
  Inherited Destroy;
end;

Procedure TDecoratable.AddDecorator(Decorator: TDecorator);
begin
  FDecorators.Add(Decorator);
end;

Function TDecoratable.GetVar;
begin
  Result := FSharedVars[Name];
  //Log.PrintLn(IntToStr(Ord(Result = Nil)));
  //  If Result = Nil then
  //    Result := NullTypeClass;
end;

Procedure TDecoratable.SendEvent(Event: TEvent);
begin
  FDecorators.SendEvent(Event);
end;

Function TDecoratable.HasDecorator(DecType: TClassEntity): Boolean;
  Var
    I: Integer;
begin
  Result := False;
    For I := 0 to FDecorators.High do
      If FDecorators[I] is DecType then
        Result := True;
end;

Function TDecoratable.HasDecorator(DecType: String): Boolean;
begin
  Result := FindDecorator(DecType) <> Nil;
end;

Function TDecoratable.FindDecorator(DecType: String): TDecorator;
  Var
    I: Integer;
begin
  Result := Nil;
    For I := 0 to FDecorators.High do
      If AnsiLowerCase(FDecorators[I].ClassName) = AnsiLowerCase(DecType) then
      begin
        Result := TDecorator(FDecorators[I]);
        Exit;
      end;
end;
//}}}
//  TFactory//{{{
//
Procedure TFactory.RegistClass(TheClass: TClassEntity);
begin
  SetLength(FClasses, Length(FClasses)+1);
  FClasses[High(FClasses)] := TheClass;
  Log.Println('TFactory.RegistClass(' + TheClass.ClassName + ')');
end;

Function TFactory.CreateEntity(TypeName: String; Owner: TEntity): TEntity;
  Var
    I: Integer;
begin
  Result := Nil;
    For I := 0 to High(FClasses) do
      If AnsiLowerCase(TypeName) = 
         AnsiLowerCase(FClasses[I].ClassName) then
        Result := FClasses[I].Create(Owner);
end;
//}}}

// TEventQue//{{{
constructor TEventQue.Create;//{{{
begin
  inherited Create(Owner);
  InitCriticalSection(FCritical);
  New(FHead);
  FHead^.Event := nil;
  FHead^.Next := FHead;
  ClearAll;
end;
//}}}
destructor TEventQue.Destroy;//{{{
begin
  ClearAll;
  DoneCriticalSection(FCritical);
  inherited Destroy;
end;
//}}}
procedure TEventQue.ClearAll;//{{{
  var
    Item, Next: PEventQueItem;
begin
  EnterCriticalSection(FCritical);
  Item := FHead^.Next;
  while Item <> FHead do begin
    Next := Item^.Next;
    Item^.Event.Free;
    Dispose(Item);
    Item := Next;
  end;
  FHead^.Next := FHead;
  FLast := FHead;
  LeaveCriticalSection(FCritical);
end;
//}}}
procedure TEventQue.Que;//{{{
  var
    Item: PEventQueItem;
begin
  EnterCriticalSection(FCritical);
  New(Item);
  Item^.Event := Event;
  Item^.Next := FHead;
  FLast^.Next := Item;
  FLast := Item;
  LeaveCriticalSection(FCritical);
end;
//}}}
function TEventQue.EnQue: TEvent;//{{{
  var
    FStart: PEventQueItem;
begin
  EnterCriticalSection(FCritical);
  FStart := FHead^.Next;
  if FStart <> FHead then begin
    if FStart = FLast then
      FLast := FHead;
    Result := FStart^.Event;
    FHead^.Next := FStart^.Next;
    Dispose(FStart);
  end else
    Result := nil;
  LeaveCriticalSection(FCritical);
end;
//}}}
function TEventQue.EnQue(var Event: TEvent): Boolean;//{{{
begin
  Event := EnQue;
  Result := Event <> nil;
end;

//}}}
function TEventQue.EnQueAll;//{{{
begin
end;
//}}}
//}}}

initialization
  Factory := TFactory.Create;
  NullTypeClass := TTypeClass.Create(Nil);
  Factory.RegistClass(TUniVar);
  Factory.RegistClass(TIntegerClass);
  Factory.RegistClass(TStringClass);
finalization
  NullTypeClass.Free;
  Factory.Free;
end.
