unit DParser;

interface

Uses
  DDebug,
  DUtils,
  DCore;

Type
  TArrayOfString = Array Of String;

Function DeleteLeftSpaces(S: String): String;
Function DeleteRightSpaces(S: String): String;
Function StringDeletePre(S: String; Count: Integer): String;
Function StringDeletePost(S: String; Start: Integer): String;

// S = List[0] B List[1] B ... B List[n-1]
Procedure ParseList(S: String; B: String; Var List: TArrayOfString);
// S = LBR
Function ParseBinary(S, B: String; Var L, R: String): Boolean;

//   FileName     List
//    ,   
// <id>: <x>
//    
// List.SetLength(Max(List.Length, id+1)); 
// List[id] := <x>;
Procedure ReadNumberedStringList(FileName: String; List: TListOfString);

//
//      ,     
//  Decoratable.
//  @param Owner -  .    
//     TEvCreated
//  @param Decoratable - ,    
//  @param S - ,   .
//    Decoratable.
//
//    S   :
//  <DecName>!s = <D1>!s1, <D2>!s2, ..., <Dn>!sn
//  DecName -   (   DoString('name=<DecName>')), 
//            s -   (Decoratable.DoString(s))  
//  D1 -   , s1 -    
//  D2 -   , s2 -    
//  ...
//  Dk -  k- , sk -    
//
Function ParseDecoratable(Owner: TEntity; 
                          Decoratable: TDecoratable;
                          S: String): TDecoratable;

implementation

Function DeleteLeftSpaces(S: String): String;
begin
  Result := S;
    While (Length(Result) > 0) And (Result[1] in [#0..#32]) do
      Delete(Result, 1, 1);
end;

Function DeleteRightSpaces(S: String): String;
begin
  Result := S;
    While (Length(Result) > 0) And (Result[Length(Result)] in [#0..#32]) do
      Delete(Result, Length(Result), 1);
end;

Function StringDeletePre(S: String; Count: Integer): String;
begin
  Result := S;
    If Count = 0 then
      Exit;
  Delete(Result, 1, Count);
end;

Function StringDeletePost(S: String; Start: Integer): String;
begin
  Result := S;
    If Start = 0 then
      Exit;
  Delete(Result, Start, Length(Result) - Start + 1);
end;

Procedure ParseList(S: String; B: String; Var List: TArrayOfString);
begin
  SetLength(List, 1);
    While Pos(B, S) <> 0 do begin
      //List[High(List)] := StringDeletePost(S, Pos(B, S));
      //S := StringDeletePre(S, Pos(B, S) + Length(B) - 1);
      //SetLength(List, Length(List) + 1);
      ParseBinary(S, B, List[High(List)], S);
      SetLength(List, Length(List) + 1);
    end;
  List[High(List)] := S;
end;

Function ParseBinary;
begin
  Result := Pos(B, S) <> 0;
  L := StringDeletePost(S, Pos(B, S));
  R := StringDeletePre(S, Pos(B, S) + Length(B) - 1);
end;

Procedure ReadNumberedStringList(FileName: String; List: TListOfString);
  Var
    F: TextFile;
    S, SID, Script: String;
    ID: Integer;
begin
  Assign(F, FileName);
  {$I-}
  Reset(F);
  {$I+}
    If IOResult <> 0 then begin
      Log.PrintLn('DParser.ReadNumberedStringList: file "' + FileName + '" not found.');
      Exit;
    end;
    While Not Eof(F) do begin
      Readln(F, S);
      ParseBinary(S, ':', SID, Script);
        If SID = Script then
          Continue;
      ID := StrToInt(SID);
        If ID < 0 then
          Continue;
        If ID > List.High then
          List.SetLength(ID+1);
      List[ID] := DeleteLeftSpaces(DeleteRightSpaces(Script));
    end;
  Close(F);
end;

Function ParseDecoratable;
  Var
    Entity: TEntity;
    L, R: String;
    I: Integer;
    DecList: TArrayOfString;
    DecoratableName, EntityClass, Params: String;
begin
  Result := Decoratable;
    If S = '' then
      Exit;
  ParseBinary(S, '|', L, R);
  //L := StringDeletePre(L, 1);
  L := DeleteLeftSpaces(DeleteRightSpaces(L));
  R := DeleteLeftSpaces(DeleteRightSpaces(R));
  ParseList(R, ',', DecList);

  ParseBinary(L, '!', DecoratableName, Params);
  DecoratableName := DeleteLeftSpaces(DeleteRightSpaces(DecoratableName));
  Result.DoString('name=' + DecoratableName);
  Result.DoString(Params);
    For I := 0 to High(DecList) do begin
      ParseBinary(DecList[I], '!', EntityClass, Params);
      EntityClass := DeleteLeftSpaces(DeleteRightSpaces(EntityClass));
      Entity := Factory.CreateEntity(EntityClass, Result);
        If (Entity = Nil) Or Not(Entity is TDecorator) then begin
          Log.PrintLn('Error[ParseDecoratable]: decorator ' + DecList[I] +
                      ' not created');
        end else begin
          Result.AddDecorator(TDecorator(Entity));
          Owner.SendEventAndFree(TEvCreated.Create(Owner, Entity));
          Entity.DoString(Params);
        end;
    end;
end;

end.
