{
        TODO:        Data
        TODO:    
}
unit ResBMP;

interface

Uses
  DUtils,
  DDebug,
  
  ResCore,
  ResImage;

Type
TBMPLoader = class(TLoader)
private
  Function LoadMono(Data: TData; Image: TImage; 
                    Width, Height: Integer): Boolean;
  Function LoadRGB4(Data: TData; Image: TImage; 
                     Width, Height: Integer;
                     Palette: Array Of Cardinal): Boolean;
  Function LoadRGB8(Data: TData; Image: TImage; 
                     Width, Height: Integer;
                     Palette: Array Of Cardinal): Boolean;
  Function LoadRGB24(Data: TData; Image: TImage; 
                     Width, Height: Integer): Boolean;
  Function LoadRGB32(Data: TData; Image: TImage; 
                     Width, Height: Integer): Boolean;
public
  Function Load(FileName: String; Data: TData): TResource; override;
end;

implementation

Const
        // BMP specific constants and structs
        BI_RGB  = 0;
        BI_RLE8 = 1;
        BI_RLE4 = 2;

Type
// structs of BMP file
TBmpHeader = packed record
  Typ: Word;                    // type of file, must be 'BM'
  Size: Cardinal;               // size of file in bytes
  Reserved1: Word;
  Reserved2: Word;
  OffBits: Cardinal;            // offset from this header to actual data
end;

TBmpInfoHeader = packed record
  Size: Cardinal;               // sizeof of this header
  Width: Cardinal;                  	// width of bitmap in pixels
  Height: Cardinal;                 	// height of bitmap in pixels
  Planes: Word;                 	// # of planes
  BitCount: Word;               	// bits per pixel
  Compression: Cardinal;            	// type of compression, BI_RGB - no compression
  SizeImage: Cardinal;              	// size of image in bytes
  XPelsPerMeter: Cardinal;          	// hor. resolution of the target device
  yPelsPerMeter: Cardinal;          	// vert. resolution
  clrUsed: Cardinal;
  clrImportant: Cardinal;
end;

TRGBQuad = packed record
  Blue: Byte;
  Green: Byte;
  Red: Byte;
  Reserved: Byte;
end;

//
//  BMP Mono
//
Function TBmpLoader.LoadMono;
  Var
    BytesPerLine: Integer;
    Delta: Integer;
    Bytes: PArrayOfByte;
    P: Pointer;
    X, Y: Integer;
begin
  Result := False;
  GetMem(P, Width*Height*4);
  BytesPerLine := (Width + 7) div 8;
  Delta := BytesPerLine And 3;
    If delta <> 0 then
      BytesPerLine := BytesPerLine + 4 - delta;

  GetMem(Bytes, BytesPerLine);
    For Y := Height-1 downto 0 do begin
      Data.GetVar(Bytes, BytesPerLine);
        //If Data.SeekCur(BytesPerLine) = Data.Len then begin
        //  Exit;
        //end;
        For X := 0 to Width-1 do
          If (Bytes^[X shr 3] And ($80 shr (X And 7))) > 0 then begin
          //If (Bytes^[X shr 3] And (1 shl (X And 7))) > 0 then begin
            TArrayOfByte(P^)[(Abs(Y*Width) + X)*4 + 0] := 255;
            TArrayOfByte(P^)[(Abs(Y*Width) + X)*4 + 1] := 255;
            TArrayOfByte(P^)[(Abs(Y*Width) + X)*4 + 2] := 255;
            TArrayOfByte(P^)[(Abs(Y*Width) + X)*4 + 3] := 255;
	  end else begin
            TArrayOfByte(P^)[(Abs(Y*Width) + X)*4 + 0] := 0;
            TArrayOfByte(P^)[(Abs(Y*Width) + X)*4 + 1] := 0;
            TArrayOfByte(P^)[(Abs(Y*Width) + X)*4 + 2] := 0;
            TArrayOfByte(P^)[(Abs(Y*Width) + X)*4 + 3] := 0;
          end;
    end;
  Image.Init(Width, Height, 4, P);
  Result := True;
end;

//
//  BMP 4bits
//
Function TBmpLoader.LoadRGB4;
  Type
    T = Array[0..3] Of Byte;
  Var
    B, Count: Integer;
    X, Y: Integer;
    P: Pointer;
begin
  Result := False;
  GetMem(P, Width*Height*3);
    For Y := Height - 1 to 0 do begin
      Count := 0;
        For X := 0 downto Width-1 do begin
	     If (X And 1) = 0 then begin
               B := Data.GetByte;
               Move(Palette[B shr 4], TArrayOfByte(P^)[(Abs(Y*Width)+X)*3], 3);
             end else
               Move(Palette[B And $F], TArrayOfByte(P^)[(Abs(Y*Width)+X)*3], 3);
          Inc(Count);
        end;
    
      Count := (Width + 1) div 2;
        While Count mod 4 > 0 do begin
	  Inc(Count);
          Data.GetByte;
        end;
     end;
  Image.Init(Width, Height, 3, P);
  Result := True;
end;

//
//
//
Function TBmpLoader.LoadRGB8;
  Type
    T = Array[0..3] Of Byte;
  Var
    B, Count: Integer;
    X, Y: Integer;
    P: Pointer;
begin
  Result := False;
  Log.PrintlN('BMP 8bits');
  GetMem(P, Width*Height*3);
    For Y := Height-1 downto 0 do begin
      Count := 0;
        For X := 0 to Width-1 do begin
	  B := Data.GetByte;
          TArrayOfByte(P^)[(Abs(Y*Width) + X)*3 + 0] := T(Palette[B])[0];
          TArrayOfByte(P^)[(Abs(Y*Width) + X)*3 + 1] := T(Palette[B])[1];
          TArrayOfByte(P^)[(Abs(Y*Width) + X)*3 + 2] := T(Palette[B])[2];
          //TArrayOfByte(P^)[(Abs(Y*Width) + X)*4 + 3] := T(Palette[B])[3];
          //Move(Palette[B], TArrayOfByte(P^)[(Abs(Y*Width) + X)*3], 3);
          Inc(Count);
        end;

        While Count mod 4 > 0 do begin
	  Inc(Count);
          Data.GetByte;
        end;
     end;
  Image.Init(Width, Height, 3, P);
  Result := True;
end;

//
//  Load RGB 24
//
Function TBmpLoader.LoadRGB24;
  Var
    Count: Integer;
    //Red, Green, Blue: Integer;
    X, Y: Integer;
    Buf: Array Of Cardinal;
    P: Pointer;
begin
  GetMem(P, Width*Height*3);
  SetLength(Buf, Width);
    For Y := height-1 downto 0 do begin
      //memset ( buf, '\0', sizeof ( dword ) * width );
      Count := 0;
        For X := 0 to width - 1 do begin
          //Blue := Data.GetByte;
          //Green := Data.GetByte;
          //Red := Data.GetByte;
          //Buf[X] := makeRgba(Red, Green, Blue);
          TArrayOfByte(P^)[(Abs(Y*Width) + X)*3 + 2] := Data.GetByte;
          TArrayOfByte(P^)[(Abs(Y*Width) + X)*3 + 1] := Data.GetByte;
          TArrayOfByte(P^)[(Abs(Y*Width) + X)*3    ] := Data.GetByte;
          Inc(Count, 3);
        end;
        While Count mod 4 > 0 do begin    // skip remaining bytes
          Data.GetByte;
          Inc(Count);
        end;
       //-> putLine ( y, buf );
       //Move(Buf[0], TArrayOfByte(P^)[Abs(Y*Width)], Width*3);
       //Move(TArrayOfByte(P^)[Abs(Y*Width)], Buf[0], Width*3);
    end;
  Image.Init(Width, Height, 3, P);
  Result := True;
end;

//
//  LoadRGB 32
//
Function TBMPLoader.LoadRGB32;
  Var
    X, Y: Integer;
    P: Pointer;
begin
  Result := False;
  GetMem(P, Width*Height*4);
    For Y := Height - 1 downto 0 do begin
        For X := 0 to Width - 1 do begin
          TArrayOfByte(P^)[(Abs(Y*Width) + X)*4 + 2] := Data.GetByte;
          TArrayOfByte(P^)[(Abs(Y*Width) + X)*4 + 1] := Data.GetByte;
          TArrayOfByte(P^)[(Abs(Y*Width) + X)*4    ] := Data.GetByte;
          TArrayOfByte(P^)[(Abs(Y*Width) + X)*4 + 3] := Data.GetByte; 
        end;
    end;
  Image.Init(Width, Height, 4, P);
  Result := True;
end;

//
//  Main load
//
Function TBMPLoader.Load;
  Type
    TQ = Array[0..1] Of TRGBQuad;
    PQ = ^TQ;
  Var
    Hdr: TBmpHeader;
    InfoHdr: TBmpInfoHeader;
    ErrorCode: Boolean;
    NumColors: Integer;
    Width: Integer;
    Height: Integer;
    Pal: PQ;
    Palette: Array[0..255] Of Cardinal;
    //NumComponents: Integer;
    Buf: Array Of Cardinal;
    I: Integer;
    Image: TImage;
begin
  Result := Inherited Load(FileName, Data);
    If Pos('.bmp', AnsiLowerCase(FileName)) <> Length(FileName)-3 then begin
      Exit;
    end;
    If (Data = Nil)Or(Not Data.Ready) then
      Exit;
  Data.GetVar(@Hdr, SizeOf(TBmpHeader));
  Data.GetVar(@InfoHdr, SizeOf(TBmpInfoHeader));
  ErrorCode := (Hdr.Typ = $4D42)And(InfoHdr.Size = 40)And
               (InfoHdr.Planes = 1)And
                ((InfoHdr.BitCount = 1)Or(InfoHdr.BitCount = 2)Or
                (InfoHdr.BitCount = 4)Or(InfoHdr.BitCount = 8)Or 
                (InfoHdr.BitCount = 24)Or(InfoHdr.BitCount = 32)) And
               ((InfoHdr.Compression = BI_RGB)Or
                (InfoHdr.Compression = BI_RLE4)Or
                (InfoHdr.Compression = BI_RLE8));

    If Not ErrorCode then                    // bad header buf
      Exit;

  NumColors := 1 shl InfoHdr.BitCount;
  Width := InfoHdr.Width;
  Height := InfoHdr.Height;

  Pal := Data.GetPtr;
  //NumComponents := 3;

  //  If InfoHdr.BitCount = 32 then
  //    NumComponents := 4;

     If InfoHdr.BitCount <= 8 then
       //Data.GetVar(@Palette[0], NumColors*SizeOf(TRGBQuad));
       For I := 0 to NumColors - 1 do begin
         Palette[I] := MakeRgba(Pal^[I].Red, Pal^[I].Green, Pal^[I].Blue);
       end;

  Image := TImage.Create(Owner); 
  //Image.Init(Width, Height, NumComponents, ...);

    If Image = Nil then
      Exit;

  SetLength(Buf, Width);

  Data.SeekAbs(Hdr.OffBits);

    If InfoHdr.Compression = BI_RGB then begin 
        If InfoHdr.BitCount = 1 then  // mono
          ErrorCode := LoadMono(Data, Image, Width, Height)
        else
        If InfoHdr.bitCount = 4 then  // 16-colors uncompressed
          ErrorCode := LoadRGB4(Data, Image, Width, Height, Palette)
        else
        If InfoHdr.BitCount = 8 then // 256-colors uncompressed
          ErrorCode := LoadRGB8 (Data, Image, Width, Height, Palette)
        else
        If InfoHdr.BitCount = 24 then // True-Color bitmap
    	  ErrorCode := LoadRGB24(Data, Image, Width, Height)
   	else
        If InfoHdr.BitCount = 32 then // true-color bitmap with alpha-channel
      	  ErrorCode := LoadRGB32(Data, Image, Width, Height)
    end else
    If InfoHdr.Compression = BI_RLE4 then// 16-colors RLE compressed
      //ErrorCode := loadRLE4 ( data, tex, buf, palette );
    else
    If InfoHdr.Compression = BI_RLE8 then// 256-colors RLE compressed
      //ErrorCode := loadRLE8 ( data, tex, buf, palette );
    else
      ErrorCode := false;

    If Not ErrorCode then begin
      Image.Free;
      Exit;
    end;
  Log.PrintLn('Bmp file "' + FileName + '" ' + 
              IntToStr(Width) + 'x' + IntToStr(Height) + ' ' +
              IntToStr(InfoHdr.BitCount) + ' loaded.');
  Result := Image;
end;

initialization
  ImgMan.RegistLoader(TBMPLoader.Create(ImgMan));
end.
