unit Use_OpenGL;

interface

uses Windows, Messages, OpenGl;

type
  PRGB = record        {      }
   r,g,b:byte;
  end;
  TSline = array[0..1] of PRGB;   { }

function IntToStr(X:integer):string;
procedure InitGL(var ghRC:HGLRC;var ghDC:HDC;var Hand:THandle;W,H:Integer);
procedure On_Resize(Width,Height:integer);
procedure CloseGL(ghRC:HGLRC;ghDC:HDC;Hand:HWND);
procedure enable_bind_tex;
procedure disable_bind_tex;
procedure Put_Sprite(X,Y,M:single;f:single;r,g,b,a:single);overload;
procedure Put_Sprite(X,Y,M:single;f:single);overload;
procedure Put_Sprite(X,Y,A,M:single;f:single);overload;
procedure Change_Resolution(x, y: word);     
procedure Restore_Resolution;
procedure Put_Tile(X,Y,M:single;i,j:byte;f:single);
function Point_To_Coord(Point:TVector3f):TPoint;

implementation

function IntToStr(X:integer):string;
begin
 Str(X, result);
end;

function Point_To_Coord(Point:TVector3f):TPoint;
Var
X,Y,Z:double;
viewport:TVector4i;
projection,modelview:TMatrix4d;
begin
{  GLdouble vx,vy,vz;       //       viewport-a.
{  GLdouble wx,wy,wz;       //   .
}
 glGetIntegerv(GL_VIEWPORT,@viewport);           //   viewport-a.
 glGetDoublev(GL_PROJECTION_MATRIX,@projection); //   .
 glGetDoublev(GL_MODELVIEW_MATRIX,@modelview);   //   .
 gluProject(Point[0],Point[1],Point[2],modelview,projection,viewport,@X,@Y,@Z);
  Point_To_Coord.X:=Round(X);
  Point_To_Coord.Y:=Round(Y);
end;

procedure Change_Resolution(x, y: word); { }
var
  dm: TDEVMODE;
begin
  ZeroMemory(@dm, sizeof(TDEVMODE));
  dm.dmSize := sizeof(TDEVMODE);
  dm.dmPelsWidth := x;
  dm.dmPelsHeight := y;
  dm.dmFields := DM_PELSWIDTH or DM_PELSHEIGHT;
  ChangeDisplaySettings(dm, 0);
end;

procedure Restore_Resolution;
begin
 ChangeDisplaySettings(devMode(nil^), 0);{ }
end;

procedure Put_Sprite(X,Y,M:single;f:single;r,g,b,a:single); overload;
begin
{       ,   ,}
{ f    RGB      Modulate       }
 glcolor4f(r,g,b,a);        { }
 glbegin(GL_Quads);         {  }
  gltexcoord2f(0,0);
  glvertex2f(-f/2+x*M,-f/2+y*M);    {}
  gltexcoord2f(1,0);
  glvertex2f(+f/2+x*M,-f/2+y*M);
  gltexcoord2f(1,1);        {}
  glvertex2f(+f/2+x*M,+f/2+y*M);
  gltexcoord2f(0,1);
  glvertex2f(-f/2+x*M,+f/2+y*M);
 glend;
end;

procedure Put_Sprite(X,Y,M:single;f:single);overload;
begin
{       ,   ,}
{ f    RGB      Modulate       }
 glbegin(GL_Quads);         {  }
  gltexcoord2f(0,0);
  glvertex2f(-f/2+x*M,-f/2+y*M);    {}  
  gltexcoord2f(1,0);
  glvertex2f(+f/2+x*M,-f/2+y*M);
  gltexcoord2f(1,1);        {}
  glvertex2f(+f/2+x*M,+f/2+y*M);
  gltexcoord2f(0,1);
  glvertex2f(-f/2+x*M,+f/2+y*M);
 glend;
end;

procedure Put_Sprite(X,Y,A,M:single;f:single);overload;
begin
{       ,   ,}
{ f    RGB      Modulate       }
 glTranslatef(x*M,y*M,0);
 glRotatef(a,0,0,1);
 glbegin(GL_Quads);         {  }
  gltexcoord2f(0,0);
  glvertex2f(-f/2,-f/2);    {}
  gltexcoord2f(1,0);
  glvertex2f(+f/2,-f/2);
  gltexcoord2f(1,1);        {}
  glvertex2f(+f/2,+f/2);
  gltexcoord2f(0,1);
  glvertex2f(-f/2,+f/2);
 glend;
 glRotatef(-a,0,0,1);
 glTranslatef(-x*M,-y*M,0);
end;

procedure Put_Tile(X,Y,M:single;i,j:byte;f:single);
begin
 glbegin(GL_Quads);
  gltexcoord2f(i*0.25+0.002,j*0.25+0.002);
  glvertex2f(-f/2+x*M,-f/2+y*M);    {}
  gltexcoord2f(i*0.25+0.25-0.002,j*0.25+0.002);
  glvertex2f(+f/2+x*M,-f/2+y*M);
  gltexcoord2f(i*0.25+0.25-0.002,j*0.25+0.25-0.002);
  glvertex2f(+f/2+x*M,+f/2+y*M);
  gltexcoord2f(i*0.25+0.002,j*0.25+0.25-0.002);
  glvertex2f(-f/2+x*M,+f/2+y*M);
 glend;
{}
end;

procedure On_Resize(Width,Height:integer); {    }
begin
 glViewport(0,0,Width,Height);        
 glMatrixMode(GL_Projection);
 glLoadIdentity();
 gluPerspective(45.0, Width/Height, 0.1, 100.0);{  }
 glOrtho(-20,20,-20,20,0,50);
 gluLookAt(0,0,12, 0,0,0, 0,1,0);
 glMatrixMode(Gl_ModelView);
end;



function SetupPixelFormat(DC:HDC;g:byte):boolean;
var
pfd:PIXELFORMATDESCRIPTOR;
ppfd:PPIXELFORMATDESCRIPTOR;
pixelformat:integer;
begin
ppfd := @pfd;
ppfd.nSize := sizeof(PIXELFORMATDESCRIPTOR);
ppfd.nVersion := 1;
ppfd.dwFlags :=  PFD_DRAW_TO_WINDOW xor
                           PFD_SUPPORT_OPENGL xor
                           PFD_DOUBLEBUFFER;
ppfd.dwLayerMask := PFD_MAIN_PLANE;
ppfd.iPixelType := PFD_TYPE_RGBA;
ppfd.cColorBits := g;
ppfd.cDepthBits := g;
ppfd.cAccumBits := 0;
ppfd.cStencilBits := 1;
pixelformat := ChoosePixelFormat(dc, ppfd);
if pixelformat=0 then
begin
MessageBox(0, 'ChoosePixelFormat failed', 'Error', MB_OK);
SetupPixelFormat:=FALSE;
exit;
end;

if SetPixelFormat(dc, pixelformat, ppfd)=false then
begin
MessageBox(0, 'SetPixelFormat failed', 'Error', MB_OK);
SetupPixelFormat:=FALSE;
exit;
end;
SetupPixelFormat:=TRUE;
end;

procedure InitGL(var ghRC:HGLRC;var ghDC:HDC;var Hand:THandle;W,H:Integer);
Var
 p:TVector4f;
 d:TVector3f;
begin   {OpenGL}
 ghDC:=GetDC(HAND);       { }
 If SetupPixelFormat(ghDC,16)=False then
  Exit;
 ghRC:=wglCreateContext(ghDC);
 wglMakeCurrent(ghDC,ghRC);            {   }
 glClearColor(0.0,0.0,0.0,0.0);        {  - }
 On_Resize(W,H);                       {  }
 glEnable(GL_Color_Material);
 glEnable(GL_Depth_Test);              {Z }
 glEnable(GL_Lighting);                { }
 glEnable(GL_LIGHT0);                  { 0}
 glenable(GL_Point_Smooth);            { }
 p[0]:=2; p[1]:=-5; p[2]:=3; p[3]:=-2;      { }
 d[0]:=0; d[1]:=0; d[2]:=0;
 glLightFV(Gl_LIGHT0,Gl_Position,@p);          { }
 glLightFV(GL_LIGHT0,GL_Spot_Direction,@d);
 p[0]:=0.4; p[1]:=0.4; p[2]:=0.4; p[3]:=1;      { }
 glLightFV(Gl_LIGHT0,GL_AMBIENT,@p);          { }
 {Settings}
 glDepthFunc(GL_LESS);		           //  Z 
 glHint(GL_PERSPECTIVE_CORRECTION_HINT, GL_NICEST);   //
end;

procedure CloseGL(ghRC:HGLRC;ghDC:HDC;Hand:HWND);
begin
 If ghRC<>0 then             { ,  }
  begin
  wglMakeCurrent(ghDc,0);
  wglDeleteContext(ghRC);
 end;
 If ghDC<>0 then
  ReleaseDC(Hand,ghDC);
end;

procedure enable_bind_tex;
begin
 glEnable(GL_TEXTURE_2D);         { }
 glEnable(GL_BLEND);             { }
 glBlendFunc(GL_SRC_ALPHA, GL_ONE_MINUS_SRC_ALPHA);{ }
end;

procedure disable_bind_tex;
begin
 glDisable(GL_BLEND);      { }
 glDisable(GL_TEXTURE_2D); { }
end;

end.
