#include <object.h>
#include <core.h>

#include <hge.h>
#include <hgeresource.h>
#include <hgesprite.h>
#include <hgefont.h>
#include <hgeparticle.h>

CBall::CBall(CCore *engine, HGE *Hge) : CGameObject(engine,hge) {
	Timer = 0.f;
	x=100.0f;
	y=100.0f;
	dx=0.0f;
	dy=0.0f;	
	speed=90;
    friction=0.98f;
	Engine=engine;
	hge = Hge;
}

CBall::~CBall(void) {
	Engine = 0;
	// Delete created objects and free loaded resources
	delete par;
	delete fnt;
	delete spt;
	delete spr;
	Engine->hge->Texture_Free(tex);
	Engine->hge->Effect_Free(snd);
	hge=0;
}

bool CBall::Init(void) {
		// Load sound and texture
		snd=Engine->hge->Effect_Load("menu.wav");
		tex=Engine->hge->Texture_Load("data\graphics\particles.png");
		if(!snd || !tex)
		{
			// If one of the data files is not found, display
			// an error message and shutdown.
			MessageBox(NULL, "Can't load one of the following files:\nMENU.WAV, PARTICLES.PNG, FONT1.FNT, FONT1.PNG, TRAIL.PSI", "Error", MB_OK | MB_ICONERROR | MB_APPLMODAL);
			Engine->hge->System_Shutdown();
			Engine->hge->Release();
			return false;
		}

		// Create and set up a sprite
		spr=new hgeSprite(tex, 96, 64, 32, 32);
		spr->SetColor(0xFFFFA000);
		spr->SetHotSpot(16,16);

		// Load a font
		fnt=new hgeFont("font1.fnt");

		// Create and set up a particle system
		spt=new hgeSprite(tex, 32, 32, 32, 32);
		spt->SetBlendMode(BLEND_COLORMUL | BLEND_ALPHAADD | BLEND_NOZWRITE);
		spt->SetHotSpot(16,16);
		par=new hgeParticleSystem("trail.psi",spt);
		par->Fire();

	return true;
}

void CBall::boom() {
	int pan=int((x-400)/4);
	float pitch=(dx*dx+dy*dy)*0.0005f+0.2f;
	Engine->hge->Effect_PlayEx(snd,100,pan,pitch);
}
void CBall::Update(float delta) {
	// Process keys
	if (Engine->hge->Input_GetKeyState(HGEK_LEFT)) dx-=speed*delta;
	if (Engine->hge->Input_GetKeyState(HGEK_RIGHT)) dx+=speed*delta;
	if (Engine->hge->Input_GetKeyState(HGEK_UP)) dy-=speed*delta;
	if (Engine->hge->Input_GetKeyState(HGEK_DOWN)) dy+=speed*delta;

	// Do some movement calculations and collision detection	
	dx*=friction; dy*=friction; x+=dx; y+=dy;
	if(x>784) {x=784-(x-784);dx=-dx;boom();}
	if(x<16) {x=16+16-x;dx=-dx;boom();}
	if(y>584) {y=584-(y-584);dy=-dy;boom();}
	if(y<16) {y=16+16-y;dy=-dy;boom();}

	// Update particle system
	par->info.nEmission=(int)(dx*dx+dy*dy)*2;
	par->MoveTo(x,y);
	par->Update(delta);

	Timer += delta;
	if(Timer > 1.f) {
		Timer -= 1.f;
	//
	}
}

void CBall::Render(void) {
	// Render graphics
	par->Render();
	spr->Render(x, y);
}